package ParallaxUSBOscilloscope;

use 5.008008;
use strict;
use warnings;
use Device::SerialPort;
use Switch;

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration   use ParallaxUSBOscilloscope ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
#our %EXPORT_TAGS = ( 'all' => [ qw(
#    
#) ] );

#our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw();

our $VERSION = '0.01';

sub new {
    my $class = shift;
    my $filename = shift || return undef;
    my $lockfile = shift;
    my $port = Device::SerialPort->new( $filename, undef, undef )
            || return undef;

    $port->baudrate(40000000);
    $port->parity("none");
    $port->databits(8);
    $port->stopbits(1);
    $port->handshake("none");

    $port->read_char_time(0);
    $port->read_const_time(100);

    my $self = {};

    $self->{'port'} = $port;
    $self->{'sampling_rate'}           = 500;
    $self->{'trigger_voltage'}         = 128;
    $self->{'trigger_edge'}            = 'rising';
    $self->{'trigger_source'}          = '1';
    $self->{'trigger_mode'}            = 'auto';
    $self->{'voltage_range_channel_1'} = '20Vpp';
    $self->{'voltage_range_channel_2'} = '20Vpp';
    $self->{'high_speed_channel'}      = '1';
    $self->{'disconnected'} = undef;

    return bless ($self, $class);
}

sub voltage_range {
    my $arg_count = scalar(@_);
    my $self = shift;
    my $channel = shift;
    my $range = shift;

    if( 3 == $arg_count ) {
        return undef unless( $channel =~ /^[12]$/ );
        return undef unless( $range =~ /^(20|4)Vpp$/ );

        $self->{"voltage_range_channel_$channel"} = $range;
    }

    return ($self->{'voltage_range_channel_1'},
            $self->{'voltage_range_channel_2'});
}

sub sampling_rate {
    my $arg_count = scalar(@_);
    my $self = shift;
    my $sampling_rate = shift;

    if( 2 == $arg_count ) {
        switch( $sampling_rate ) {
            case [50,100,250,500,1000,2500,5000,10000,25000,50000,100000,250000,500000,1000000]
            {
                $self->{'sampling_rate'} = int($sampling_rate);
            } else {
                return undef;
            }
        }
    }

    return $self->{'sampling_rate'};
}

sub ping {
    my $self = shift;
    my $port = $self->{'port'};
    my %code_to_month = ( "1" => "Jan", "2" => "Feb", "3" => "Mar",
                          "4" => "Apr", "5" => "May", "6" => "Jun",
                          "7" => "Jul", "8" => "Aug", "9" => "Sep",
                          "A" => "Oct", "B" => "Nov", "C" => "Dec" );

    if( 1 == $port->write(chr(0x3f)) ) {

        my ($count, $data) = $port->read( 255 );

        if( 7 <= $count ) {
            my @chars;

            @chars = split( //, $data );

            my $prProductFamily;
            my $prModel;
            my $prFirmwareRevision;
            my $prManufactureMonth;
            my $prManufactureYear;
            my $manufactureDate;

            $prProductFamily = $chars[0];
            $prModel = $chars[1];
            $prFirmwareRevision = "$chars[2].$chars[3].$chars[4]";
            $prManufactureMonth = $code_to_month{$chars[5]};
            $prManufactureYear = 2001 + int($chars[6]);
            $manufactureDate = $prManufactureMonth.', '.$prManufactureYear;

            if( $prProductFamily =~ m/[O]/ ) {
                return ($prModel, $prFirmwareRevision, $manufactureDate);
            }
        } else {
            if( (!defined $count) && (!defined $data) ) {
                $self->{'disconnected'} = 'true';
            }
        }
    }

    return undef;
}

sub abort {
    my $self = shift;
    my $port = $self->{'port'};
    my $model = undef;
    my $firmware = undef;
    my $date = undef;

    return undef unless (1 == $port->write(chr(0xaf)));

    do {
        ($model, $firmware, $date) = $self->ping();
        if( defined $self->{'disconnected'} ) {
            return undef;
        }
    } while( !defined $model );

    return "true";
}

sub acquire {
    my $self = shift;
    my $port = $self->{'port'};
    my $start = sprintf( "%c", 0x85 );
    my @b = ();
    $b[0] = 0xaa;
    $b[1] = $self->{'trigger_voltage'};
    $b[2] = 1;  # The document says the selected channel must always be 1. */
    if( $self->{'voltage_range_channel_2'} =~ /20Vpp/    ) { $b[2] |= 0x80; }
    if( $self->{'voltage_range_channel_1'} =~ /20Vpp/    ) { $b[2] |= 0x40; }
    if( $self->{'trigger_edge'           } =~ /rising/   ) { $b[2] |= 0x30; }
    $b[3] = 0;
    if( $self->{'trigger_source'         } =~ /2/        ) { $b[3] |= 0x01; }
    if( $self->{'trigger_mode'           } =~ /auto/     ) { $b[3] |= 0x04; }
    switch( $self->{'sampling_rate'} ) {
        case 50         { $b[3] |= 0x68; }
        case 100        { $b[3] |= 0x60; }
        case 250        { $b[3] |= 0x58; }
        case 500        { $b[3] |= 0x50; }
        case 1000       { $b[3] |= 0x48; }
        case 2500       { $b[3] |= 0x40; }
        case 5000       { $b[3] |= 0x38; }
        case 10000      { $b[3] |= 0x30; }
        case 25000      { $b[3] |= 0x28; }
        case 50000      { $b[3] |= 0x20; }
        case 100000     { $b[3] |= 0x18; }
        case 250000     { $b[3] |= 0x10; }
        case 500000     { $b[3] |= 0x08; }
        case 1000000    { $b[3] |= 0x08; }
    }
    $b[4] = 0;
    if( $self->{'trigger_source'         } =~ /external/ ) { $b[4] |= 0x40; }
    if( $self->{'sampling_rate'          } == 1000000    ) {
    print "here\n";
        if( $self->{'high_speed_channel' } =~ /1/        ) {
            $b[4] |= 0x18;
        } else {
            $b[4] |= 0x28;
        }
    }
    $b[5] = 240;    # Pre Trigger Value     LSB 1520    MSB 61445   0xF005
    $b[6] = 5;
    $b[7] = 39;     # Post Trigger Value    LSB 807     MSB 9987    0x2703
    $b[8] = 3;
    my $command = sprintf( "%c%c%c%c%c%c%c%c%c", $b[0], $b[1], $b[2],
                           $b[3], $b[4], $b[5], $b[6], $b[7], $b[8] );

    # Send the command
    return undef unless (scalar(@b) == $port->write($command));

    {
        my $count = undef;
        my $data = undef;

        # Find the starting 0x85 character
        do {
            ($count, $data) = $port->read( 1 );
            if( (!defined $count) && (!defined $data) ) {
                $self->{'disconnected'} = 'true';
                return undef;
            }
        } while( (1 != $count) && ($start ne $data) );
    }

    # Read the rest of the data
    my @samples = ();

    {
        my $offset = 0;
        my $count = undef;
        my $data = undef;

        do {
            ($count, $data) = $port->read( 255 );
            if( (!defined $count) && (!defined $data) ) {
                $self->{'disconnected'} = 'true';
                return undef;
            }

            if( 0 < $count ) {
                my @tmp;
                @tmp = split( //, $data );
                foreach my $i (@tmp) {
                    $samples[$offset] = $i;
                    $offset++;
                }
            }
        } while( 0 < $count );
    }

    my @channel_1 = ();
    my @channel_2 = ();
    my @timestamp = ();

    # Process the data into the Left and Right Channels if needed
    if( $self->{'sampling_rate'} == 1000000 ) {
        my $scale_factor = 0.0156;
        my $channel = $self->{'high_speed_channel'};
        if( $self->{"voltage_range_channel_$channel"} =~ /20Vpp/ ) {
            $scale_factor = 0.0781;
        }
        if( $channel =~ /1/ ) {
            for( my $i = 0; $i < scalar(@samples); $i++ ) {
                $channel_1[$i] = (ord($samples[$i]) - 128) * $scale_factor;
            }
        } else {
            for( my $i = 0; $i < scalar(@samples); $i++ ) {
                $channel_2[$i] = (ord($samples[$i]) - 128) * $scale_factor;
            }
        }

        my $delta = 1 / int($self->{'sampling_rate'});
        for( my $i = 0; $i < scalar(@samples); $i++ ) {
            $timestamp[$i] = $i * $delta;
        }
    } else {
        my $mid = scalar(@samples) / 2;
        my $scale_factor = 0.0156;
        if( $self->{'voltage_range_channel_2'} =~ /20Vpp/ ) { $b[4] |= 0x40; }
        for( my $i = 0; $i < $mid; $i++ ) {
            $channel_2[$i] = (ord($samples[$i]) - 128) * $scale_factor;
        }

        $scale_factor = 0.0156;
        if( $self->{'voltage_range_channel_1'} =~ /20Vpp/ ) { $b[4] |= 0x40; }
        for( my $i = $mid; $i < scalar(@samples); $i++ ) {
            $channel_1[$i - $mid] = (ord($samples[$i]) - 128) * $scale_factor;
        }

        my $delta = 1 / int($self->{'sampling_rate'});
        for( my $i = 0; $i < scalar(@channel_1); $i++ ) {
            $timestamp[$i] = $i * $delta;
        }
    }

    return (\@timestamp, \@channel_1, \@channel_2);
}

# Preloaded methods go here.

1;  # so the require or use succeeds

__END__

=head1 NAME

Device::ParallaxUSBOscilloscope - Parallax USB Oscilloscope controller

=head1 SYNOPSIS

 use ParallaxUSBOscilloscope;

 # open the device
 my $scope = ParallaxUSBOscilloscope->new( $DeviceName, $lockfile )
         || die "Can't open $DeviceName: $!\n";

=head1 DESCRIPTION

Device::ParallaxUSBOscilloscope is a control module for the Parallax USB
Oscilloscope.

=head2 FUNCTIONS

=head3 voltage_range

    @channel_voltage_ranges = $scope->voltage_range( $channel, $range )
    @channel_voltage_ranges = $scope->voltage_range()

Sets or gets the voltage range for each channel.  Only one channel
at a time is set.  All channels are returned on a successful set
or a get.

Valid parameters:
    $channel - '1' or '2' selecting the channel to set
    $range - '20Vpp' or '4Vpp' selects the range for the channel

Returns:
    On success, the currently set voltage ranges for all the channels.
    The size of the array is 2.  Channel 1 is index 0, Channel 2
    is index 1.
    On failure, undef.

=head3 trigger_config

TBD

=head3 sampling_rate

    $sampling_rate = $scope->sampling_rate( $rate )
    $sampling_rate = $scope->sampling_rate()

Sets or gets the sampling rate (number of samples per second).  Both
channels use the same sampling rate.  If the sample rate of 1000000
is selected, only the channel selected as the 'high speed channel'
will be sampled.  The sampling rate is returned on a successful set
or a get.

Valid parameters:
    $rate - 50, 100, 250, 500, 1000, 2500, 5000, 10000, 25000,
            50000, 100000, 250000, 500000, 1000000

Returns:
    On success, the currently set sampling rate (samples per second).
    On failure, undef.


=head3 ping

    ($model, $firmware, $manufactureDate) = $scope->ping()

Used to see if the device is still operating, and to get device
hardware information.

Returns:
    $model - the string represenging the device model
    $firmware - 'x.y.z' string of the current firmware
    $manufactureDate - 'mon, year' formatted date
    On failure, undef is returned


=head3 abort

    $success = $scope->abort()

Used to stop any current action and put the device back into the
idle state.

Returns:
    On success, defined.
    On failure, undef.

=head3 acquire

    ($timestamp_ref, $channel_1_ref, $channel_2_ref) = $scope->acquire()

Used to use the current settings to get a set of samples.

Returns:
    $timestamp_ref - a reference to an array of sample times 
                     (in seconds)
    $channel_1_ref - a reference to an array of channel 1 voltages
                     (in volts)
    $channel_2_ref - a reference to an array of channel 1 voltages
                     (in volts)
    On failure, undef is returned

=head2 EXPORT

none

=head1 SEE ALSO

http://parascope.sourceforge.net

=head1 AUTHOR

Weston T. Schmidt, E<lt>weston_schmidt@alumni.purdue.eduE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2007 by Weston T. Schmidt

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.


=cut
